package com.example.service;

import com.azure.core.http.rest.PagedIterable;
import com.azure.data.appconfiguration.ConfigurationClient;
import com.azure.data.appconfiguration.models.ConfigurationSetting;
import com.azure.data.appconfiguration.models.SettingSelector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This Class is used to fetch from Azure Cloud App Configuration or set properties to Azure Cloud App Configuration
 * Before running we need to set Connection String into ENV variable, so that our application will connect to Azure Cloud Portal
 */
public class AzureAppConfigService {

    private static final Logger LOGGER = LoggerFactory.getLogger(AzureAppConfigService.class);
    private final ConfigurationClient configurationClient;
    private final SettingSelector settingSelector;

    public AzureAppConfigService(ConfigurationClient configurationClient, SettingSelector settingSelector) {
        this.configurationClient = configurationClient;
        this.settingSelector = settingSelector;
    }

    /**
     * To Fetch Value based on Key from Azure App Configuration
     * @param key String
     * @return the value based on key
     */
    public String getValueByKey(String key) {
        try {
            LOGGER.info("Read Single Value by key");
            ConfigurationSetting configurationSetting = configurationClient.getConfigurationSetting(key, null);
            return configurationSetting.getValue();
        } catch (Exception e) {
            LOGGER.error(e.getMessage());
            return null;
        }
    }

    /**
     * To Send Value with respect to Key to Azure App Configuration
     * @param key String
     * @param value String
     * @return the value based on key
     */
    public String writeValueByKey(String key, String value) {
        try {
            LOGGER.info("Writing Single Value by key");
            ConfigurationSetting configurationSetting = configurationClient.setConfigurationSetting(key, null, value);
            return configurationSetting.getValue();
        } catch (Exception e) {
            LOGGER.error(e.getMessage());
            return null;
        }
    }

    /**
     * To Fetch All Values from Azure App Configuration
     * @return all values
     */
    public PagedIterable<ConfigurationSetting> readAllKeyValue() {
        LOGGER.info("Read All Values...");
        return configurationClient.listConfigurationSettings(settingSelector);
    }

    /**
     * To Delete Value based on Key from Azure App Configuration
     * @param key String
     * @return the value based on key
     */
    public String deleteValueByKey(String key) {
        try {
            LOGGER.info("Remove Single Value by key");
            ConfigurationSetting configurationSetting = configurationClient.deleteConfigurationSetting(key, null);
            return configurationSetting.getValue();
        } catch (Exception e) {
            LOGGER.error(e.getMessage());
            return null;
        }
    }
}
